@@ -339,7 +339,7 @@ impl DrawingCtx {
     pub fn acquire_node(
         &mut self,
         fragment: &Fragment,
-        node_types: &[NodeType]
+        node_types: &[NodeType],
     ) -> Result<AcquiredNode, AcquireError> {
         self.num_elements_acquired += 1;
 
@@ -350,15 +350,17 @@ impl DrawingCtx {
         self.acquired_nodes.acquire(fragment, node_types)
     }
 
+    pub fn acquire_node_ref(&mut self, node: &RsvgNode) -> Result<AcquiredNode, AcquireError> {
+        self.acquired_nodes.push_node_ref(node)
+    }
+
     // Returns (clip_in_user_space, clip_in_object_space), both Option<RsvgNode>
     fn get_clip_in_user_and_object_space(
         &mut self,
         clip_uri: Option<&Fragment>,
     ) -> (Option<RsvgNode>, Option<RsvgNode>) {
         clip_uri
-            .and_then(|fragment| {
-                self.acquire_node(fragment, &[NodeType::ClipPath]).ok()
-            })
+            .and_then(|fragment| self.acquire_node(fragment, &[NodeType::ClipPath]).ok())
             .and_then(|acquired| {
                 let clip_node = acquired.get().clone();
 
@@ -486,8 +488,7 @@ impl DrawingCtx {
                     // Mask
 
                     if let Some(fragment) = mask {
-                        if let Ok(acquired) = dc.acquire_node(fragment, &[NodeType::Mask])
-                        {
+                        if let Ok(acquired) = dc.acquire_node(fragment, &[NodeType::Mask]) {
                             let mask_node = acquired.get();
 
                             res = res.and_then(|bbox| {
@@ -545,7 +546,14 @@ impl DrawingCtx {
         let matrix = self.cr.get_matrix();
         let res = draw_fn(self);
         self.cr.set_matrix(matrix);
-        res
+
+        if let Ok(bbox) = res {
+            let mut orig_matrix_bbox = BoundingBox::new(&matrix);
+            orig_matrix_bbox.insert(&bbox);
+            Ok(orig_matrix_bbox)
+        } else {
+            res
+        }
     }
 
     /// Saves the current Cairo context, runs the draw_fn, and restores the context
@@ -567,8 +575,7 @@ impl DrawingCtx {
         child_surface: &cairo::ImageSurface,
         node_bbox: BoundingBox,
     ) -> Result<cairo::ImageSurface, RenderingError> {
-        match self.acquire_node(filter_uri, &[NodeType::Filter])
-        {
+        match self.acquire_node(filter_uri, &[NodeType::Filter]) {
             Ok(acquired) => {
                 let filter_node = acquired.get();
 
@@ -600,16 +607,16 @@ impl DrawingCtx {
 
     fn set_color(
         &self,
-        color: &cssparser::Color,
-        opacity: &UnitInterval,
-        current_color: &cssparser::RGBA,
+        color: cssparser::Color,
+        opacity: UnitInterval,
+        current_color: cssparser::RGBA,
     ) {
-        let rgba = match *color {
-            cssparser::Color::RGBA(ref rgba) => rgba,
+        let rgba = match color {
+            cssparser::Color::RGBA(rgba) => rgba,
             cssparser::Color::CurrentColor => current_color,
         };
 
-        let &UnitInterval(o) = opacity;
+        let UnitInterval(o) = opacity;
         self.get_cairo_context().set_source_rgba(
             f64::from(rgba.red_f32()),
             f64::from(rgba.green_f32()),
@@ -632,9 +639,9 @@ impl DrawingCtx {
     pub fn set_source_paint_server(
         &mut self,
         ps: &PaintServer,
-        opacity: &UnitInterval,
+        opacity: UnitInterval,
         bbox: &BoundingBox,
-        current_color: &cssparser::RGBA,
+        current_color: cssparser::RGBA,
     ) -> Result<bool, RenderingError> {
         match *ps {
             PaintServer::Iri {
@@ -672,7 +679,7 @@ impl DrawingCtx {
                 }
 
                 if !had_paint_server && alternate.is_some() {
-                    self.set_color(alternate.as_ref().unwrap(), opacity, current_color);
+                    self.set_color(alternate.unwrap(), opacity, current_color);
                     had_paint_server = true;
                 } else {
                     rsvg_log!(
@@ -685,7 +692,7 @@ impl DrawingCtx {
             }
 
             PaintServer::SolidColor(color) => {
-                self.set_color(&color, opacity, current_color);
+                self.set_color(color, opacity, current_color);
                 Ok(true)
             }
 
@@ -733,12 +740,10 @@ impl DrawingCtx {
         // coordinate system in patterns.
         let bbox = compute_stroke_and_fill_box(cr, values);
 
-        let current_color = &values.color.0;
-
-        let fill_opacity = &values.fill_opacity.0;
+        let current_color = values.color.0;
 
         let res = self
-            .set_source_paint_server(&values.fill.0, fill_opacity, &bbox, current_color)
+            .set_source_paint_server(&values.fill.0, values.fill_opacity.0, &bbox, current_color)
             .and_then(|had_paint_server| {
                 if had_paint_server {
                     if values.stroke.0 == PaintServer::None {
@@ -751,13 +756,11 @@ impl DrawingCtx {
                 Ok(())
             })
             .and_then(|_| {
-                let stroke_opacity = values.stroke_opacity.0;
-
                 self.set_source_paint_server(
                     &values.stroke.0,
-                    &stroke_opacity,
+                    values.stroke_opacity.0,
                     &bbox,
-                    &current_color,
+                    current_color,
                 )
                 .and_then(|had_paint_server| {
                     if had_paint_server {
@@ -1067,19 +1070,24 @@ impl From<RsvgRectangle> for cairo::Rectangle {
     }
 }
 
-pub struct AcquiredNode(Rc<RefCell<NodeStack>>, RsvgNode);
+pub struct AcquiredNode {
+    stack: Option<Rc<RefCell<NodeStack>>>,
+    node: RsvgNode,
+}
 
 impl Drop for AcquiredNode {
     fn drop(&mut self) {
-        let mut stack = self.0.borrow_mut();
-        let last = stack.pop().unwrap();
-        assert!(last == self.1);
+        if let Some(ref stack) = self.stack {
+            let mut stack = stack.borrow_mut();
+            let last = stack.pop().unwrap();
+            assert!(last == self.node);
+        }
     }
 }
 
 impl AcquiredNode {
     pub fn get(&self) -> &RsvgNode {
-        &self.1
+        &self.node
     }
 }
 
@@ -1096,7 +1104,11 @@ impl AcquiredNodes {
         }
     }
 
-    fn lookup_node(&self, fragment: &Fragment) -> Result<AcquiredNode, AcquireError> {
+    fn lookup_node(
+        &self,
+        fragment: &Fragment,
+        node_types: &[NodeType],
+    ) -> Result<RsvgNode, AcquireError> {
         let node = self.svg.lookup(fragment).map_err(|_| {
             // FIXME: callers shouldn't have to know that get_node() can initiate a file load.
             // Maybe we should have the following stages:
@@ -1109,12 +1121,15 @@ impl AcquiredNodes {
             AcquireError::LinkNotFound(fragment.clone())
         })?;
 
-        if self.node_stack.borrow().contains(&node) {
-            Err(AcquireError::CircularReference(fragment.clone()))
+        if node_types.is_empty() {
+            Ok(node)
         } else {
-            self.node_stack.borrow_mut().push(&node);
-            let acquired = AcquiredNode(self.node_stack.clone(), node.clone());
-            Ok(acquired)
+            let node_type = node.borrow().get_type();
+            if node_types.iter().find(|&&t| t == node_type).is_some() {
+                Ok(node)
+            } else {
+                Err(AcquireError::InvalidLinkType(fragment.clone()))
+            }
         }
     }
 
@@ -1123,19 +1138,42 @@ impl AcquiredNodes {
         fragment: &Fragment,
         node_types: &[NodeType],
     ) -> Result<AcquiredNode, AcquireError> {
-        let acquired = self.lookup_node(fragment)?;
+        let node = self.lookup_node(fragment, node_types)?;
 
-        if node_types.len() == 0 {
-            Ok(acquired)
+        if node_is_accessed_by_reference(&node) {
+            self.push_node_ref(&node)
         } else {
-            let acquired_type = acquired.get().borrow().get_type();
-            if node_types.iter().find(|&&t| t == acquired_type).is_some() {
-                Ok(acquired)
-            } else {
-                Err(AcquireError::InvalidLinkType(fragment.clone()))
-            }
+            Ok(AcquiredNode {
+                stack: None,
+                node: node.clone(),
+            })
         }
     }
+
+    fn push_node_ref(&self, node: &RsvgNode) -> Result<AcquiredNode, AcquireError> {
+        if self.node_stack.borrow().contains(&node) {
+            Err(AcquireError::CircularReference(node.clone()))
+        } else {
+            self.node_stack.borrow_mut().push(&node);
+            Ok(AcquiredNode {
+                stack: Some(self.node_stack.clone()),
+                node: node.clone(),
+            })
+        }
+    }
+}
+
+// Returns whether a node of a particular type is only accessed by reference
+// from other nodes' atributes.  The node could in turn cause other nodes
+// to get referenced, potentially causing reference cycles.
+fn node_is_accessed_by_reference(node: &RsvgNode) -> bool {
+    use NodeType::*;
+
+    match node.borrow().get_type() {
+        ClipPath | Filter | LinearGradient | Marker | Mask | Pattern | RadialGradient => true,
+
+        _ => false,
+    }
 }
 
 /// Keeps a stack of nodes and can check if a certain node is contained in the stack
